@@ -986,7 +986,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 					list_for_each_entry(top, &ifp->tracking_vrrp, e_list) {
 						vrrp = top->obj.vrrp;
 
-						if (vrrp->track_saddr && vrrp->family == ifa->ifa_family)
+						if (__test_bit(VRRP_FLAG_TRACK_SADDR, &vrrp->flags) && vrrp->family == ifa->ifa_family)
 							is_tracking_saddr = inaddr_equal(ifa->ifa_family, &vrrp->saddr, addr.addr);
 						else
 							is_tracking_saddr = false;
@@ -999,7 +999,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 						    vrrp->num_script_if_fault &&
 						    vrrp->family == ifa->ifa_family &&
 						    vrrp->saddr.ss_family == AF_UNSPEC &&
-						    (!vrrp->saddr_from_config || is_tracking_saddr)) {
+						    (!__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags) || is_tracking_saddr)) {
 							/* Copy the address */
 							if (ifa->ifa_family == AF_INET)
 								inet_ip4tosockaddr(addr.in, &vrrp->saddr);
@@ -1014,7 +1014,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 							 vrrp->ifp &&
 							 ifp == vrrp->ifp->base_ifp &&
 							 IS_MAC_IP_VLAN(vrrp->ifp) &&
-							 !__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags) &&
+							 !__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->flags) &&
 							 ifa->ifa_family == AF_INET6 &&
 							 vrrp->ifp->is_ours) {
 								inet_ip6tosockaddr(addr.in6, &vrrp->saddr);
@@ -1028,7 +1028,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 									 */
 									if (add_link_local_address(vrrp->ifp, addr.in6) &&
 									    vrrp->num_script_if_fault &&
-									    (!vrrp->saddr_from_config || is_tracking_saddr))
+									    (!__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags) || is_tracking_saddr))
 										try_up_instance(vrrp, false);
 								} else
 #endif
@@ -1054,7 +1054,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 							vrrp = top->obj.vrrp;
 							if (VRRP_CONFIGURED_IFP(vrrp) != ifp)
 								continue;
-							if (vrrp->family != AF_INET || vrrp->saddr_from_config)
+							if (vrrp->family != AF_INET || __test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags))
 								continue;
 							inet_ip4tosockaddr(&ifp->sin_addr, &vrrp->saddr);
 						}
@@ -1081,7 +1081,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 							vrrp = top->obj.vrrp;
 							if (vrrp->ifp != ifp && ifp != vrrp->ifp->base_ifp)
 								continue;
-							if (vrrp->family != AF_INET6 || vrrp->saddr_from_config)
+							if (vrrp->family != AF_INET6 || __test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags))
 								continue;
 							inet_ip6tosockaddr(&ifp->sin6_addr, &vrrp->saddr);
 #ifdef _HAVE_VRRP_VMAC_
@@ -1130,17 +1130,17 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 					if (!inaddr_equal(ifa->ifa_family, vrrp->family == AF_INET ? &(PTR_CAST(struct sockaddr_in, &vrrp->saddr))->sin_addr : (void *)&(PTR_CAST(struct sockaddr_in6, &vrrp->saddr))->sin6_addr, addr.addr))
 						continue;
 
-					is_tracking_saddr = vrrp->track_saddr &&
+					is_tracking_saddr = __test_bit(VRRP_FLAG_TRACK_SADDR, &vrrp->flags) &&
 							    vrrp->family == ifa->ifa_family &&
 							    inaddr_equal(ifa->ifa_family, &vrrp->saddr, addr.addr);
 #ifdef _HAVE_VRRP_VMAC_
 					/* If we are a VMAC and took this address from the parent interface, we need to
 					 * release the address and create one for ourself */
 					if (ifa->ifa_family == AF_INET6 &&
-					    __test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags) &&
+					    __test_bit(VRRP_VMAC_BIT, &vrrp->flags) &&
 					    ifp == vrrp->ifp->base_ifp &&
-					    !__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags) &&
-					    !vrrp->saddr_from_config) {
+					    !__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->flags) &&
+					    !__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags)) {
 // This is rubbish if base i/f addr changed. Check against address generated from base i/f's MAC
 						if (IF_ISUP(ifp) && replace_link_local_address(vrrp->ifp))
 						{
@@ -1162,7 +1162,7 @@ netlink_if_address_filter(__attribute__((unused)) struct sockaddr_nl *snl, struc
 							 vrrp->ifp) &&
 						 vrrp->family == ifa->ifa_family &&
 						 vrrp->saddr.ss_family != AF_UNSPEC &&
-						 (!vrrp->saddr_from_config || is_tracking_saddr)) {
+						 (!__test_bit(VRRP_FLAG_SADDR_FROM_CONFIG, &vrrp->flags) || is_tracking_saddr)) {
 						down_instance(vrrp);
 						vrrp->saddr.ss_family = AF_UNSPEC;
 					}
@@ -1810,6 +1810,8 @@ netlink_if_link_populate(interface_t *ifp, struct rtattr *tb[], struct ifinfomsg
 				is_vrf = true;
 				ifp->if_type = IF_TYPE_VRF;
 				parse_rtattr_nested(vrf_attr, IFLA_VRF_MAX, linkinfo[IFLA_INFO_DATA]);
+				if (vrf_attr[IFLA_VRF_TABLE])
+					ifp->vrf_tb_id = *PTR_CAST(uint32_t, RTA_DATA(vrf_attr[IFLA_VRF_TABLE]));
 			}
 #endif
 		}
