@@ -54,6 +54,7 @@
 #include "libipvs.h"
 #endif
 #include "notify.h"
+#include "sockaddr.h"
 
 /* constants */
 #define DEFAULT_SMTP_CONNECTION_TIMEOUT (30 * TIMER_HZ)
@@ -106,7 +107,7 @@ typedef struct _data {
 #endif
 	const char			*router_id;
 	const char			*email_from;
-	struct sockaddr_storage		smtp_server;
+	sockaddr_t			smtp_server;
 	const char			*smtp_helo_name;
 	unsigned long			smtp_connection_to;
 	list_head_t			email;
@@ -145,8 +146,8 @@ typedef struct _data {
 	int				max_auto_priority;
 	long				min_auto_priority_delay;
 #ifdef _WITH_VRRP_
-	struct sockaddr_in6		vrrp_mcast_group6 __attribute__((aligned(__alignof__(struct sockaddr_storage))));
-	struct sockaddr_in		vrrp_mcast_group4 __attribute__((aligned(__alignof__(struct sockaddr_storage))));
+	struct sockaddr_in6		vrrp_mcast_group6 __attribute__((aligned(__alignof__(sockaddr_t))));
+	struct sockaddr_in		vrrp_mcast_group4 __attribute__((aligned(__alignof__(sockaddr_t))));
 	unsigned			vrrp_garp_delay;
 	timeval_t			vrrp_garp_refresh;
 	unsigned			vrrp_garp_rep;
@@ -217,6 +218,7 @@ typedef struct _data {
 	notify_fifo_t			notify_fifo;
 #ifdef _WITH_VRRP_
 	notify_fifo_t			vrrp_notify_fifo;
+	bool				fifo_write_vrrp_states_on_reload;
 #endif
 #ifdef _WITH_LVS_
 	notify_fifo_t			lvs_notify_fifo;
