@@ -20,20 +20,20 @@
  * Copyright (C) 2016-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
-#ifndef VRRP_DBUS_H
-#define VRRP_DBUS_H
+#ifndef _VRRP_DBUS_H
+#define _VRRP_DBUS_H
 
 /* System includes */
 #include <stdbool.h>
 
 /* Local includes */
 #include "vrrp.h"
-#include "list.h"
+#include "list_head.h"
 
 
 void dbus_send_state_signal(vrrp_t *);
 void dbus_remove_object(const vrrp_t *);
-void dbus_reload(const list, const list);
+void dbus_reload(const list_head_t *, const list_head_t *);
 bool dbus_start(void);
 void dbus_stop(void);
 #ifdef THREAD_DUMP
