@@ -45,6 +45,7 @@
 #include "scheduler.h"
 #include "list_head.h"
 #include "timer.h"
+#include "sockaddr.h"
 
 #define LINK_UP   1
 #define LINK_DOWN 0
@@ -57,7 +58,7 @@
 #define LB_ETHTOOL 0x4
 #endif
 
-/* We need a default MTU in case a vrrp instance using usicast doesn't specify an interface */
+/* We need a default MTU in case a vrrp instance using unicast doesn't specify an interface */
 #define	DEFAULT_MTU	1500
 
 /* I don't know what the correct type is.
@@ -167,6 +168,7 @@ typedef struct _interface {
 #ifdef _HAVE_VRF_
 	ifindex_t		vrf_master_ifindex;	/* Only used at startup if we find i/f before master i/f */
 	struct _interface	*vrf_master_ifp;	/* VRF master interface - pointer to self if VRF master */
+	uint32_t		vrf_tb_id;		/* Table id of VRF */
 #endif
 	int			reset_arp_config;	/* Count of how many vrrps have changed arp parameters on interface */
 	bool			arp_ignore;		/* Original value of arp_ignore to be restored */
@@ -233,7 +235,7 @@ extern list_head_t garp_delay;
 /* prototypes */
 extern interface_t *if_get_by_ifindex(ifindex_t) __attribute__ ((pure));
 #ifdef _HAVE_VRRP_VMAC_
-extern interface_t * if_get_by_vmac(uint8_t, int, const interface_t *) __attribute__ ((pure));
+extern interface_t * if_get_by_vmac(uint8_t, int, const interface_t *, const u_char hw_addr[ETH_ALEN]) __attribute__ ((pure));
 #endif
 extern interface_t *get_default_if(void);
 extern interface_t *if_get_by_ifname(const char *, if_lookup_t);
@@ -254,7 +256,7 @@ extern void free_interface_queue(void);
 extern void free_old_interface_queue(void);
 extern void dump_interface_queue(FILE *, list_head_t *);
 extern void reset_interface_queue(void);
-extern int if_join_vrrp_group(sa_family_t, int *, const interface_t *);
+extern int if_join_vrrp_group(sa_family_t, int *, const interface_t *, const sockaddr_t *);
 extern int if_setsockopt_bindtodevice(int *, const interface_t *);
 extern int if_setsockopt_hdrincl(int *);
 extern int if_setsockopt_ipv6_checksum(int *);
