@@ -63,53 +63,54 @@ enum iprule_param_mask {
 
  /* types definition */
 typedef struct _ip_rule {
-	uint32_t	mask;
-	bool		invert;
-	int		family;
-	ip_address_t	*from_addr;
-	ip_address_t	*to_addr;
-	uint32_t	priority;
-	uint8_t		tos;
-	uint32_t	fwmark;
-	uint32_t	fwmask;
-	uint32_t	realms;
+	uint32_t			mask;
+	bool				invert;
+	int				family;
+	ip_address_t			*from_addr;
+	ip_address_t			*to_addr;
+	uint32_t			priority;
+	uint8_t				tos;
+	uint32_t			fwmark;
+	uint32_t			fwmask;
+	uint32_t			realms;
 #if HAVE_DECL_FRA_SUPPRESS_PREFIXLEN
-	int32_t		suppress_prefix_len;
+	int32_t				suppress_prefix_len;
 #endif
 #if HAVE_DECL_FRA_SUPPRESS_IFGROUP
-	uint32_t	suppress_group;
+	uint32_t			suppress_group;
 #endif
-	interface_t	*iif;
-#if HAVE_DECL_FRA_OIFNAME
-	interface_t	*oif;
-#endif
-	uint32_t	goto_target;
-	uint32_t	table;
-	uint8_t		action;
+	interface_t			*iif;
+	interface_t			*oif;
+	uint32_t			goto_target;
+	uint32_t			table;
+	uint8_t				action;
 #if HAVE_DECL_FRA_TUN_ID
-	uint64_t	tunnel_id;
+	uint64_t			tunnel_id;
 #endif
 #if HAVE_DECL_FRA_UID_RANGE
-	struct fib_rule_uid_range uid_range;
+	struct fib_rule_uid_range	uid_range;
 #endif
 #if HAVE_DECL_FRA_L3MDEV
-	bool		l3mdev;
+	bool				l3mdev;
 #endif
 #if HAVE_DECL_FRA_PROTOCOL
-	uint8_t		protocol;
+	uint8_t				protocol;
 #endif
 #if HAVE_DECL_FRA_IP_PROTO
-	uint8_t		ip_proto;
+	uint8_t				ip_proto;
 #endif
 #if HAVE_DECL_FRA_SPORT_RANGE
-	struct fib_rule_port_range src_port;
+	struct fib_rule_port_range	src_port;
 #endif
 #if HAVE_DECL_FRA_DPORT_RANGE
-	struct fib_rule_port_range dst_port;
+	struct fib_rule_port_range	dst_port;
 #endif
-	bool		dont_track;     /* used for virtual rules */
-	static_track_group_t *track_group;   /* used for static rules */
-	bool		set;
+	bool				dont_track;     /* used for virtual rules */
+	static_track_group_t		*track_group;   /* used for static rules */
+	bool				set;
+
+	/* linked list member */
+	list_head_t			e_list;
 } ip_rule_t;
 
 #define IPRULE_DEL 0
@@ -117,13 +118,15 @@ typedef struct _ip_rule {
 
 /* prototypes */
 extern void reinstate_static_rule(ip_rule_t *);
-extern void netlink_rulelist(list, int, bool);
-extern void free_iprule(void *);
+extern void netlink_rulelist(list_head_t *, int, bool);
+extern void free_iprule(ip_rule_t *);
+extern void free_iprule_list(list_head_t *);
 extern void format_iprule(const ip_rule_t *, char *, size_t);
-extern void dump_iprule(FILE *, const void *);
-extern void alloc_rule(list, const vector_t *, bool);
-extern void clear_diff_rules(list, list);
-extern void clear_diff_srules(void);
+extern void dump_iprule(FILE *, const ip_rule_t *);
+extern void dump_iprule_list(FILE *, const list_head_t *);
+extern void alloc_rule(list_head_t *, const vector_t *, bool);
+extern void clear_diff_rules(list_head_t *, list_head_t *);
+extern void clear_diff_static_rules(void);
 extern void reset_next_rule_priority(void);
 
 #endif
