@@ -17,37 +17,26 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _VRRP_NDISC_H
 #define _VRRP_NDISC_H
 
 /* system includes */
-#include <linux/icmpv6.h>
+#include <asm/byteorder.h>
+#include <netinet/icmp6.h>
+#include <netinet/in.h>
+#include <stdbool.h>
 
 /* local includes */
 #include "vrrp.h"
+#include "vrrp_if.h"
+#include "vrrp_ipaddress.h"
 
 /* local definitions */
-#define ETHERNET_HW_LEN	6
-#define NEXTHDR_ICMP	58
 #define NDISC_HOPLIMIT	255
 
-/*
- *	ICMPv6 codes for Neighbour Discovery messages
- */
-#define NDISC_ROUTER_SOLICITATION	133
-#define NDISC_ROUTER_ADVERTISEMENT	134
-#define NDISC_NEIGHBOUR_SOLICITATION	135
-#define NDISC_NEIGHBOUR_ADVERTISEMENT	136
-#define NDISC_REDIRECT			137
-
-/*
- *	Neighbour Discovery option codes
- */
-#define ND_OPT_TARGET_LL_ADDR	2
-
 /*
  *	IPv6 Header
  */
@@ -71,23 +60,8 @@ struct ip6hdr {
 	struct	in6_addr	daddr;
 };
 
-/*
- *	NDISC Neighbour Advertisement related
- */
-struct ndhdr {
-	struct icmp6hdr		icmph;
-	struct in6_addr		target;
-	__u8			opt[0];
-};
-
-struct nd_opt_hdr {
-	__u8			nd_opt_type;
-	__u8			nd_opt_len;
-} __attribute__((__packed__));
-
-
 /* prototypes */
-extern void ndisc_init(void);
+extern bool ndisc_init(void);
 extern void ndisc_close(void);
 extern void ndisc_send_unsolicited_na(vrrp_t *, ip_address_t *);
 extern void ndisc_send_unsolicited_na_immediate(interface_t *, ip_address_t *);
