@@ -31,6 +31,7 @@
 #include <net/if_arp.h>
 #include <linux/if_packet.h>
 #include <errno.h>
+#include <stdbool.h>
 
 /* local includes */
 #include "logger.h"
@@ -116,6 +117,9 @@ ssize_t send_gratuitous_arp_immediate(interface_t *ifp, ip_address_t *ipaddress)
 	if (ifp->hw_addr_len == 0)
 		return -1;
 
+	if (!garp_buffer)
+		return -1;
+
 	/* Setup link layer header */
 	if (ifp->hw_type == ARPHRD_INFINIBAND) {
 		struct ipoib_hdr  *ipoib;
@@ -212,24 +216,30 @@ void send_gratuitous_arp(vrrp_t *vrrp, ip_address_t *ipaddress)
 /*
  *	Gratuitous ARP init/close
  */
-void gratuitous_arp_init(void)
+bool
+gratuitous_arp_init(void)
 {
 	if (garp_buffer)
-		return;
+		return true;
 
 	/* Create the socket descriptor */
-	garp_fd = socket(PF_PACKET, SOCK_RAW | SOCK_CLOEXEC | SOCK_NONBLOCK, htons(ETH_P_RARP));
+	garp_fd = socket(PF_PACKET, SOCK_RAW | SOCK_CLOEXEC | SOCK_NONBLOCK, htons(ETH_P_ARP));
 
-	if (garp_fd >= 0) {
-		if (__test_bit(LOG_DETAIL_BIT, &debug))
-			log_message(LOG_INFO, "Registering gratuitous ARP shared channel");
-	} else {
+	if (garp_fd < 0) {
 		log_message(LOG_INFO, "Error %d while registering gratuitous ARP shared channel", errno);
-		return;
+		return (errno != EAFNOSUPPORT && errno != EPERM);
 	}
 
+	if (__test_bit(LOG_DETAIL_BIT, &debug))
+		log_message(LOG_INFO, "Registering gratuitous ARP shared channel");
+
+	/* We don't want to receive any data on this socket */
+	if_setsockopt_no_receive(&garp_fd);
+
 	/* Initalize shared buffer */
 	garp_buffer = PTR_CAST(char, MALLOC(GARP_BUFFER_SIZE));
+
+	return true;
 }
 
 void gratuitous_arp_close(void)
