@@ -152,8 +152,8 @@ vrrp_json_data_dump(json_writer_t *wr, vrrp_t *vrrp)
 
 	/* Global instance related */
 	jsonw_string_field(wr, "iname", vrrp->iname);
-	jsonw_uint_field(wr, "dont_track_primary", vrrp->dont_track_primary);
-	jsonw_uint_field(wr, "skip_check_adv_addr", vrrp->skip_check_adv_addr);
+	jsonw_uint_field(wr, "dont_track_primary", __test_bit(VRRP_FLAG_DONT_TRACK_PRIMARY, &vrrp->flags));
+	jsonw_uint_field(wr, "skip_check_adv_addr", __test_bit(VRRP_FLAG_SKIP_CHECK_ADV_ADDR, &vrrp->flags));
 	jsonw_uint_field(wr, "strict_mode", vrrp->strict_mode);
 #ifdef _HAVE_VRRP_VMAC_
 	jsonw_string_field(wr, "vmac_ifname", vrrp->vmac_ifname);
@@ -174,13 +174,13 @@ vrrp_json_data_dump(json_writer_t *wr, vrrp_t *vrrp)
 	jsonw_uint_field(wr, "base_priority", vrrp->base_priority);
 	jsonw_uint_field(wr, "effective_priority", vrrp->effective_priority);
 	jsonw_bool_field(wr, "vipset", vrrp->vipset);
-	jsonw_bool_field(wr, "promote_secondaries", vrrp->promote_secondaries);
+	jsonw_bool_field(wr, "promote_secondaries", __test_bit(VRRP_FLAG_PROMOTE_SECONDARIES, &vrrp->flags));
 	jsonw_float_field(wr, "adver_int", vrrp->adver_int / TIMER_HZ_DOUBLE);
 	jsonw_float_field(wr, "master_adver_int", vrrp->master_adver_int / TIMER_HZ_DOUBLE);
 #ifdef _WITH_FIREWALL_
 	jsonw_uint_field(wr, "accept", vrrp->accept);
 #endif
-	jsonw_bool_field(wr, "nopreempt", vrrp->nopreempt);
+	jsonw_bool_field(wr, "nopreempt", __test_bit(VRRP_FLAG_NOPREEMPT, &vrrp->flags));
 	jsonw_uint_field(wr, "preempt_delay", vrrp->preempt_delay / TIMER_HZ);
 	jsonw_uint_field(wr, "state", vrrp->state);
 	jsonw_uint_field(wr, "wantstate", vrrp->wantstate);
@@ -278,16 +278,18 @@ void
 vrrp_print_json(void)
 {
 	FILE *fp;
+	const char *filename;
 
 	if (list_empty(&vrrp_data->vrrp))
 		return;
 
-	fp = fopen_safe(KA_TMP_DIR "/keepalived.json", "w");
-	if (!fp) {
-		log_message(LOG_INFO, "Can't open " KA_TMP_DIR "/keepalived.json (%d: %m)", errno);
-		return;
-	}
+	filename = make_tmp_filename("keepalived.json");
+	fp = fopen_safe(filename, "w");
+	if (fp) {
+		vrrp_json_dump(fp);
+		fclose(fp);
+	} else
+		log_message(LOG_INFO, "Can't open %s/keepalived.json (%d: %m)", tmp_dir, errno);
 
-	vrrp_json_dump(fp);
-	fclose(fp);
+	FREE_CONST(filename);
 }
