@@ -33,6 +33,7 @@
 #include <netinet/in.h>
 #include <stdint.h>
 #include <errno.h>
+#include <stdbool.h>
 
 /* local includes */
 #include "logger.h"
@@ -40,9 +41,6 @@
 #include "vrrp_if_config.h"
 #include "vrrp_scheduler.h"
 #include "vrrp_ndisc.h"
-#if !HAVE_DECL_SOCK_CLOEXEC
-#include "old_socket.h"
-#endif
 #include "bitops.h"
 
 /* static vars */
@@ -262,31 +260,27 @@ ndisc_send_unsolicited_na(vrrp_t *vrrp, ip_address_t *ipaddress)
 /*
  *	Neighbour Discovery init/close
  */
-void
+bool
 ndisc_init(void)
 {
 	if (ndisc_fd != -1)
-		return;
+		return true;
 
 	/* Create the socket descriptor */
 	ndisc_fd = socket(PF_PACKET, SOCK_RAW | SOCK_CLOEXEC | SOCK_NONBLOCK, htons(ETH_P_IPV6));
 
-	if (ndisc_fd >= 0) {
-		if (__test_bit(LOG_DETAIL_BIT, &debug))
-			log_message(LOG_INFO, "Registering gratuitous NDISC shared channel");
-	} else {
+	if (ndisc_fd < 0) {
 		log_message(LOG_INFO, "Error %d while registering gratuitous NDISC shared channel", errno);
-		return;
+		return (errno != EAFNOSUPPORT && errno != EPERM);
 	}
 
-#if !HAVE_DECL_SOCK_CLOEXEC
-	if (set_sock_flags(ndisc_fd, F_SETFD, FD_CLOEXEC))
-		log_message(LOG_INFO, "Unable to set CLOEXEC on gratuitous NA socket");
-#endif
-#if !HAVE_DECL_SOCK_NONBLOCK
-	if (set_sock_flags(garp_fd, F_SETFL, O_NONBLOCK))
-		log_message(LOG_INFO, "Unable to set NONBLOCK on gratuitous NA socket");
-#endif
+	if (__test_bit(LOG_DETAIL_BIT, &debug))
+		log_message(LOG_INFO, "Registering gratuitous NDISC shared channel");
+
+	/* We don't want to receive any data on this socket */
+	if_setsockopt_no_receive(&ndisc_fd);
+
+	return true;
 }
 
 void
