@@ -44,7 +44,7 @@
 
 #include "scheduler.h"
 #include "memory.h"
-#include "rbtree.h"
+#include "rbtree_ka.h"
 #include "utils.h"
 #include "signals.h"
 #include "logger.h"
@@ -55,6 +55,7 @@
 #include "warnings.h"
 #include "process.h"
 #include "align.h"
+#include "systemd.h"
 
 
 #ifdef THREAD_DUMP
@@ -137,26 +138,31 @@ get_thread_type_str(thread_type_t id)
 }
 
 static inline int
-function_cmp(const func_det_t *func1, const func_det_t *func2)
+function_cmp(const void *func, const struct rb_node *a)
 {
-	if ((const void*)func1->func < (const void*)func2->func)
+	if (func < (void *)rb_entry_const(a, func_det_t, n)->func)
 		return -1;
-	if ((const void*)func1->func > (const void *)func2->func)
+	if (func > (void *)rb_entry_const(a, func_det_t, n)->func)
 		return 1;
 	return 0;
 }
 
+static inline bool
+function_less(struct rb_node *a, const struct rb_node *b)
+{
+	return (rb_entry_const(a, func_det_t, n)->func < rb_entry_const(b, func_det_t, n)->func);
+}
+
 static const char *
 get_function_name(thread_func_t func)
 {
-	func_det_t func_det = { .func = func };
-	func_det_t *match;
+	struct rb_node *match_node;
 	static char address[19];
 
 	if (!RB_EMPTY_ROOT(&funcs)) {
-		match = rb_search(&funcs, &func_det, n, function_cmp);
-		if (match)
-			return match->name;
+		match_node = rb_find((void *)func, &funcs, function_cmp);
+		if (match_node)
+			return rb_entry(match_node, func_det_t, n)->name;
 	}
 
 	snprintf(address, sizeof address, "%p", func);
@@ -185,7 +191,7 @@ register_thread_address(const char *func_name, thread_func_t func)
 	func_det->name = func_name;
 	func_det->func = func;
 
-	rb_insert_sort(&funcs, func_det, n, function_cmp);
+	rb_add(&func_det->n, &funcs, function_less);
 }
 
 void
@@ -203,10 +209,10 @@ deregister_thread_addresses(void)
 	if (RB_EMPTY_ROOT(&funcs))
 		return;
 
-	rb_for_each_entry_safe(func_det, func_det_tmp, &funcs, n) {
-		rb_erase(&func_det->n, &funcs);
+	rbtree_postorder_for_each_entry_safe(func_det, func_det_tmp, &funcs, n)
 		FREE(func_det);
-	}
+
+	funcs = RB_ROOT;
 }
 #endif
 
@@ -235,7 +241,7 @@ register_shutdown_function(void (*func)(int))
 #endif
 
 /* Move ready thread into ready queue */
-static int
+static void
 thread_move_ready(thread_master_t *m, rb_root_cached_t *root, thread_t *thread, int type)
 {
 	rb_erase_cached(&thread->n, root);
@@ -243,16 +249,18 @@ thread_move_ready(thread_master_t *m, rb_root_cached_t *root, thread_t *thread,
 	list_add_tail(&thread->e_list, &m->ready);
 	if (thread->type != THREAD_TIMER_SHUTDOWN)
 		thread->type = type;
-	return 0;
 }
 
 /* Move ready thread into ready queue */
 static void
 thread_rb_move_ready(thread_master_t *m, rb_root_cached_t *root, int type)
 {
-	thread_t *thread, *thread_tmp;
+	thread_t *thread;
+	rb_node_t *thread_node;
+
+	while ((thread_node = rb_first_cached(root))) {
+		thread = rb_entry(thread_node, thread_t, n);
 
-	rb_for_each_entry_safe_cached(thread, thread_tmp, root, n) {
 		if (thread->sands.tv_sec == TIMER_DISABLED || timercmp(&time_now, &thread->sands, <))
 			break;
 
@@ -260,6 +268,7 @@ thread_rb_move_ready(thread_master_t *m, rb_root_cached_t *root, int type)
 			thread->event->read = NULL;
 		else if (type == THREAD_WRITE_TIMEOUT)
 			thread->event->write = NULL;
+
 		thread_move_ready(m, root, thread, type);
 	}
 }
@@ -269,11 +278,12 @@ static void
 thread_update_timer(rb_root_cached_t *root, timeval_t *timer_min)
 {
 	const thread_t *first;
+	rb_node_t *first_node;
 
-	if (!root->rb_root.rb_node)
+	if (!(first_node = rb_first_cached(root)))
 		return;
 
-	first = rb_entry(rb_first_cached(root), thread_t, n);
+	first = rb_entry(first_node, thread_t, n);
 
 	if (first->sands.tv_sec == TIMER_DISABLED)
 		return;
@@ -361,9 +371,15 @@ thread_timerfd_handler(thread_ref_t thread)
 
 /* Child PID cmp helper */
 static inline int
-thread_child_pid_cmp(const thread_t *t1, const thread_t *t2)
+thread_child_pid_cmp(const void *pid, const rb_node_t *a)
 {
-	return less_equal_greater_than(t1->u.c.pid, t2->u.c.pid);
+	return less_equal_greater_than(*PTR_CAST_CONST(pid_t, pid), rb_entry_const(a, thread_t, rb_data)->u.c.pid);
+}
+
+static inline bool
+thread_child_pid_less(rb_node_t *a, const rb_node_t *b)
+{
+	return rb_entry(a, thread_t, rb_data)->u.c.pid < rb_entry_const(b, thread_t, rb_data)->u.c.pid;
 }
 
 void
@@ -449,7 +465,7 @@ log_command_line(unsigned indent)
 
 RELAX_STRICT_OVERFLOW_START
 	for (i = 0, p = log_str; i < sav_argc; i++) {
-RELAX_END
+RELAX_STRICT_OVERFLOW_END
 		p += sprintf(p, "%s'%s'", i ? " " : "", sav_argv[i]);
 	}
 
@@ -509,7 +525,7 @@ log_child_died(const char *process, pid_t pid)
 }
 
 /* report_child_status returns true if the exit is a hard error, so unable to continue */
-bool
+int
 report_child_status(int status, pid_t pid, char const *prog_name)
 {
 	char const *prog_id = NULL;
@@ -531,9 +547,19 @@ report_child_status(int status, pid_t pid, char const *prog_name)
 
 		/* Handle exit codes of vrrp or checker child */
 		if (exit_status == KEEPALIVED_EXIT_FATAL ||
-		    exit_status == KEEPALIVED_EXIT_CONFIG) {
-			log_message(LOG_INFO, "%s exited with permanent error %s. Terminating", prog_id, exit_status == KEEPALIVED_EXIT_CONFIG ? "CONFIG" : "FATAL" );
-			return true;
+		    exit_status == KEEPALIVED_EXIT_CONFIG ||
+		    exit_status == KEEPALIVED_EXIT_MISSING_PERMISSION) {
+			log_message(LOG_INFO, "%s exited with permanent error %s. Terminating",
+					prog_id,
+					exit_status == KEEPALIVED_EXIT_CONFIG ? "CONFIG" :
+					  exit_status == KEEPALIVED_EXIT_MISSING_PERMISSION ? "missing permission" :
+					  "FATAL" );
+
+#ifdef _USE_SYSTEMD_NOTIFY_
+			systemd_notify_error(exit_status == KEEPALIVED_EXIT_MISSING_PERMISSION ? EPERM : EINVAL);
+#endif
+
+			return exit_status;
 		}
 
 		if (exit_status != EXIT_SUCCESS)
@@ -576,7 +602,7 @@ report_child_status(int status, pid_t pid, char const *prog_name)
 					WTERMSIG(status) == SIGKILL ? " - has rlimit_rttime been exceeded?" : "");
 	}
 
-	return false;
+	return 0;
 }
 #endif
 
@@ -606,13 +632,17 @@ thread_events_resize(thread_master_t *m, int delta)
 }
 
 static inline int
-thread_event_cmp(const thread_event_t *event1, const thread_event_t *event2)
+thread_event_cmp(const void *key, const rb_node_t *a)
 {
-	if (event1->fd < event2->fd)
-		return -1;
-	if (event1->fd > event2->fd)
-		return 1;
-	return 0;
+	int fd = *PTR_CAST_CONST(int, key);
+
+	return fd - rb_entry_const(a, thread_event_t, n)->fd;
+}
+
+static inline bool
+thread_event_less(rb_node_t *a, const rb_node_t *b)
+{
+	return rb_entry(a, thread_event_t, n)->fd < rb_entry_const(b, thread_event_t, n)->fd;
 }
 
 static thread_event_t *
@@ -631,7 +661,7 @@ thread_event_new(thread_master_t *m, int fd)
 
 	event->fd = fd;
 
-	rb_insert_sort(&m->io_events, event, n, thread_event_cmp);
+	rb_add(&event->n, &m->io_events, thread_event_less);
 
 	return event;
 }
@@ -639,9 +669,13 @@ thread_event_new(thread_master_t *m, int fd)
 static thread_event_t * __attribute__ ((pure))
 thread_event_get(thread_master_t *m, int fd)
 {
-	thread_event_t event = { .fd = fd };
+	rb_node_t *node;
 
-	return rb_search(&m->io_events, &event, n, thread_event_cmp);
+	node = rb_find(&fd, &m->io_events, thread_event_cmp);
+
+	if (!node)
+		return NULL;
+	return rb_entry(node, thread_event_t, n);
 }
 
 static int
@@ -867,8 +901,8 @@ dump_thread_data(const thread_master_t *m, FILE *fp)
 }
 #endif
 
-/* declare thread_timer_cmp() for rbtree compares */
-RB_TIMER_CMP(thread);
+/* declare thread_timer_less() for rbtree compares */
+RB_TIMER_LESS(thread, n);
 
 /* Free all unused thread. */
 static void
@@ -937,11 +971,10 @@ thread_destroy_list(thread_master_t *m, list_head_t *l)
 static void
 thread_destroy_rb(thread_master_t *m, rb_root_cached_t *root)
 {
-	thread_t *thread, *thread_tmp;
-
-	rb_for_each_entry_safe_cached(thread, thread_tmp, root, n) {
-		rb_erase_cached(&thread->n, root);
+	thread_t *thread;
+	thread_t *thread_sav;
 
+	rbtree_postorder_for_each_entry_safe(thread, thread_sav, &root->rb_root, n) {
 		/* The following are relevant for the read and write rb lists */
 		if (thread->type == THREAD_READ ||
 		    thread->type == THREAD_WRITE) {
@@ -962,24 +995,28 @@ thread_destroy_rb(thread_master_t *m, rb_root_cached_t *root)
 
 		thread_add_unuse(m, thread);
 	}
+
+	*root = RB_ROOT_CACHED;
 }
 
 /* Cleanup master */
 void
-thread_cleanup_master(thread_master_t * m)
+thread_cleanup_master(thread_master_t * m, bool keep_children)
 {
 	/* Unuse current thread lists */
 	m->current_event = NULL;
 	thread_destroy_rb(m, &m->read);
 	thread_destroy_rb(m, &m->write);
 	thread_destroy_rb(m, &m->timer);
-	thread_destroy_rb(m, &m->child);
+	if (!keep_children)
+		thread_destroy_rb(m, &m->child);
 	thread_destroy_list(m, &m->event);
 #ifdef USE_SIGNAL_THREADS
 	thread_destroy_list(m, &m->signal);
 #endif
 	thread_destroy_list(m, &m->ready);
-	m->child_pid = RB_ROOT;
+	if (!keep_children)
+		m->child_pid = RB_ROOT;
 
 	if (m->current_thread) {
 		thread_add_unuse(m, m->current_thread);
@@ -1017,7 +1054,7 @@ thread_destroy_master(thread_master_t * m)
 	if (m->signal_fd != -1)
 		signal_handler_destroy();
 
-	thread_cleanup_master(m);
+	thread_cleanup_master(m, false);
 
 	FREE(m);
 }
@@ -1111,7 +1148,7 @@ thread_add_read_sands(thread_master_t *m, thread_func_t func, void *arg, int fd,
 	thread->sands = *sands;
 
 	/* Sort the thread. */
-	rb_insert_sort_cached(&m->read, thread, n, thread_timer_cmp);
+	rb_add_cached(&thread->n, &m->read, thread_timer_less);
 
 	return thread;
 }
@@ -1175,7 +1212,7 @@ thread_read_requeue(thread_master_t *m, int fd, const timeval_t *new_sands)
 
 	thread->sands = *new_sands;
 
-	rb_move_cached(&thread->master->read, thread, n, thread_timer_cmp);
+	rb_move_cached(&thread->n, &thread->master->read, thread_timer_less);
 }
 
 /* Adjust the timeout of a read thread */
@@ -1241,7 +1278,7 @@ thread_add_write(thread_master_t *m, thread_func_t func, void *arg, int fd, unsi
 	}
 
 	/* Sort the thread. */
-	rb_insert_sort_cached(&m->write, thread, n, thread_timer_cmp);
+	rb_add_cached(&thread->n, &m->write, thread_timer_less);
 
 	return thread;
 }
@@ -1294,7 +1331,7 @@ thread_add_timer_uval(thread_master_t *m, thread_func_t func, void *arg, unsigne
 	}
 
 	/* Sort by timeval. */
-	rb_insert_sort_cached(&m->timer, thread, n, thread_timer_cmp);
+	rb_add_cached(&thread->n, &m->timer, thread_timer_less);
 
 	return thread;
 }
@@ -1332,7 +1369,7 @@ timer_thread_update_timeout(thread_ref_t thread_cp, unsigned long timer)
 
 	thread->sands = sands;
 
-	rb_move_cached(&thread->master->timer, thread, n, thread_timer_cmp);
+	rb_move_cached(&thread->n, &thread->master->timer, thread_timer_less);
 }
 
 thread_ref_t
@@ -1375,10 +1412,10 @@ thread_add_child(thread_master_t * m, thread_func_t func, void * arg, pid_t pid,
 	}
 
 	/* Sort by timeval. */
-	rb_insert_sort_cached(&m->child, thread, n, thread_timer_cmp);
+	rb_add_cached(&thread->n, &m->child,  thread_timer_less);
 
 	/* Sort by PID */
-	rb_insert_sort(&m->child_pid, thread, rb_data, thread_child_pid_cmp);
+	rb_add(&thread->rb_data, &m->child_pid, thread_child_pid_less);
 
 	return thread;
 }
@@ -1388,7 +1425,6 @@ thread_children_reschedule(thread_master_t *m, thread_func_t func, unsigned long
 {
 	thread_t *thread;
 
-// What is this used for ??
 	set_time_now();
 	rb_for_each_entry_cached(thread, &m->child, n) {
 		thread->func = func;
@@ -1418,7 +1454,7 @@ thread_add_event(thread_master_t * m, thread_func_t func, void *arg, int val)
 
 /* Add terminate event thread. */
 static thread_ref_t
-thread_add_generic_terminate_event(thread_master_t * m, thread_type_t type, thread_func_t func)
+thread_add_generic_terminate_event(thread_master_t * m, thread_type_t type, thread_func_t func, int status)
 {
 	thread_t *thread;
 
@@ -1429,7 +1465,7 @@ thread_add_generic_terminate_event(thread_master_t * m, thread_type_t type, thre
 	thread->master = m;
 	thread->func = func;
 	thread->arg = NULL;
-	thread->u.val = 0;
+	thread->u.val = status;
 	INIT_LIST_HEAD(&thread->e_list);
 	list_add_tail(&thread->e_list, &m->event);
 
@@ -1439,13 +1475,19 @@ thread_add_generic_terminate_event(thread_master_t * m, thread_type_t type, thre
 thread_ref_t
 thread_add_terminate_event(thread_master_t *m)
 {
-	return thread_add_generic_terminate_event(m, THREAD_TERMINATE, NULL);
+	return thread_add_generic_terminate_event(m, THREAD_TERMINATE, NULL, 0);
+}
+
+thread_ref_t
+thread_add_parent_terminate_event(thread_master_t *m, int status)
+{
+	return thread_add_generic_terminate_event(m, THREAD_TERMINATE, NULL, status);
 }
 
 thread_ref_t
 thread_add_start_terminate_event(thread_master_t *m, thread_func_t func)
 {
-	return thread_add_generic_terminate_event(m, THREAD_TERMINATE_START, func);
+	return thread_add_generic_terminate_event(m, THREAD_TERMINATE_START, func, 0);
 }
 
 #ifdef USE_SIGNAL_THREADS
@@ -1524,6 +1566,7 @@ thread_cancel(thread_ref_t thread_cp)
 		break;
 	case THREAD_EVENT:
 	case THREAD_READY:
+	case THREAD_READY_TIMER:
 #ifdef USE_SIGNAL_THREADS
 	case THREAD_SIGNAL:
 #endif
@@ -1531,7 +1574,13 @@ thread_cancel(thread_ref_t thread_cp)
 	case THREAD_CHILD_TERMINATED:
 		list_del_init(&thread->e_list);
 		break;
+	case THREAD_TIMER_SHUTDOWN:
+	case THREAD_TERMINATE_START:
+	case THREAD_TERMINATE:
+		log_message(LOG_WARNING, "ERROR - thread_cancel called for THREAD_%s", thread->type == THREAD_TIMER_SHUTDOWN ? "TIMER_SHUTDOWN" : thread->type == THREAD_TERMINATE ? "TERMINATE" : "TERMINATE_START");
+		return;
 	default:
+		log_message(LOG_WARNING, "ERROR - thread_cancel called for unknown thread type %u", thread->type);
 		break;
 	}
 
@@ -1965,12 +2014,13 @@ thread_call(thread_t * thread)
 	(*thread->func) (thread);
 }
 
-void
+int
 process_threads(thread_master_t *m)
 {
 	thread_t* thread;
 	list_head_t *thread_list;
 	int thread_type;
+	int exit_code = 0;
 
 	/*
 	 * Processing the master thread queues,
@@ -2002,7 +2052,7 @@ process_threads(thread_master_t *m)
 		m->current_thread = thread;
 		thread_type = thread->type;
 
-		if (thread && thread->type == THREAD_CHILD_TIMEOUT) {
+		if (thread->type == THREAD_CHILD_TIMEOUT) {
 			/* We remove the thread from the child_pid queue here so that
 			 * if the termination arrives before we processed the timeout
 			 * we can still handle the termination. */
@@ -2025,6 +2075,8 @@ process_threads(thread_master_t *m)
 		    thread->type == THREAD_CHILD_TERMINATED ||
 		    thread->type == THREAD_TIMER_SHUTDOWN ||
 		    thread->type == THREAD_TERMINATE) {
+			exit_code = thread->u.val;
+
 			if (thread->func)
 				thread_call(thread);
 
@@ -2060,25 +2112,29 @@ process_threads(thread_master_t *m)
 
 		/* If daemon hanging event is received stop processing */
 		if (thread_type == THREAD_TERMINATE)
-			break;
+			return exit_code;
 	}
+
+	return 0;
 }
 
 static void
 process_child_termination(pid_t pid, int status)
 {
 	thread_master_t * m = master;
-	thread_t th = { .u.c.pid = pid };
+	rb_node_t *thread_node;
 	thread_t *thread;
 
-	thread = rb_search(&master->child_pid, &th, rb_data, thread_child_pid_cmp);
+	thread_node = rb_find(&pid, &master->child_pid, thread_child_pid_cmp);
+	if (thread_node)
+		thread = rb_entry(thread_node, thread_t, rb_data);
 
 #ifdef _EPOLL_DEBUG_
 	if (do_epoll_debug)
-		log_message(LOG_INFO, "Child %d terminated with status 0x%x, thread_id %lu", pid, (unsigned)status, thread ? thread->id : 0);
+		log_message(LOG_INFO, "Child %d terminated with status 0x%x, thread_id %lu", pid, (unsigned)status, thread_node ? thread->id : 0);
 #endif
 
-	if (!thread)
+	if (!thread_node)
 		return;
 
 	rb_erase(&thread->rb_data, &master->child_pid);
@@ -2137,13 +2193,13 @@ thread_add_base_threads(thread_master_t *m,
 }
 
 /* Our infinite scheduling loop */
-void
+int
 launch_thread_scheduler(thread_master_t *m)
 {
 // TODO - do this somewhere better
 	signal_set(SIGCHLD, thread_child_handler, m);
 
-	process_threads(m);
+	return process_threads(m);
 }
 
 #ifdef THREAD_DUMP
